@@ -35,6 +35,20 @@ fn main()
 	}).collect::<Vec<String>>().join("-");
 	let git_id_path = Path::new(&out_dir).join("generated_git_id");
 	fs::write(&git_id_path,id_str).expect("failed to write git_id");
+
+	// alternatively use https://stackoverflow.com/questions/27840394/how-can-a-rust-program-access-metadata-from-its-cargo-package
+	let grep_version=Command::new("grep")
+		.arg("version")
+		.arg("Cargo.toml")
+		.output();
+	let version : String = grep_version.ok().and_then(|output|{
+		let version_line = String::from_utf8_lossy(&output.stdout).trim().to_string();
+		if let Some( (_left,right) ) = version_line.split_once('=') {
+			Some(right.trim().trim_matches(|c| c=='\"' || c=='\'').to_string())
+		} else { None }
+	}).unwrap_or_else(||"?".to_string());
+	let version_number_path = Path::new(&out_dir).join("generated_version_number");
+	fs::write(&version_number_path,version).expect("failed to write version number");
 	//println!("cargo:rerun-if-changed=build.rs");
 }
 
