@@ -100,13 +100,53 @@ impl CandidateEgress
 	}
 }
 
+///The candidates as provided by the routing together with related information.
+///This is, the return type of `Routing::next`.
+#[derive(Clone,Debug,Default)]
+pub struct RoutingNextCandidates
+{
+	///The vector of candidates.
+	pub candidates: Vec<CandidateEgress>,
+	///Whether sucessive calls to the routing algorithm will find the exact same set of candidates.
+	///If a call returns a `RoutingNextCandidates` with some value of `idempotent` then successive calls should also have that same value of `idempotent`.
+	///Returning `idempotent` to false allows to change the `candidates` in another call but this field should be kept to false.
+	///Setting this flag to true allows the [Router][crate::router::Router] to skip calls to the routing algorithm or even to skip some events of the router.
+	pub idempotent: bool,
+}
+
+impl From<RoutingNextCandidates> for Vec<CandidateEgress>
+{
+	fn from(candidates: RoutingNextCandidates) -> Self
+	{
+		candidates.candidates
+	}
+}
+
+impl IntoIterator for RoutingNextCandidates
+{
+	type Item = CandidateEgress;
+	type IntoIter = <Vec<CandidateEgress> as IntoIterator>::IntoIter;
+	fn into_iter(self) -> <Self as IntoIterator>::IntoIter
+	{
+		self.candidates.into_iter()
+	}
+}
+
+impl RoutingNextCandidates
+{
+	pub fn len(&self)->usize
+	{
+		self.candidates.len()
+	}
+}
+
 ///A routing algorithm to provide candidate routes when the `Router` requires.
 ///It may store/use information in the RoutingInfo.
 ///A `Routing` does not receive information about the state of buffers or similar. Such a mechanism should be given as a `VirtualChannelPolicy`.
 pub trait Routing : Debug
 {
 	///Compute the list of allowed exits.
-	fn next(&self, routing_info:&RoutingInfo, topology:&dyn Topology, current_router:usize, target_server:usize, num_virtual_channels:usize, rng: &RefCell<StdRng>) -> Vec<CandidateEgress>;
+	fn next(&self, routing_info:&RoutingInfo, topology:&dyn Topology, current_router:usize, target_server:usize, num_virtual_channels:usize, rng: &RefCell<StdRng>) -> RoutingNextCandidates;
 	//fn initialize_routing_info(&self, routing_info:&mut RoutingInfo, topology:&dyn Topology, current_router:usize, target_server:usize);
 	///Initialize the routing info of the packet. Called when the first phit of the packet leaves the server and enters a router.
 	fn initialize_routing_info(&self, routing_info:&RefCell<RoutingInfo>, topology:&dyn Topology, current_router:usize, target_server:usize, rng: &RefCell<StdRng>);
@@ -166,6 +206,8 @@ UpDown{
 }
 ```
 
+There is a `Mindless` routing without parameters that includes all neighbours as candidates until reaching destination. Can be though as a random walk, if additionally the router would make its decisions randomly.
+
 ## Operations
 
 ### Sum
@@ -335,7 +377,7 @@ pub struct Shortest
 
 impl Routing for Shortest
 {
-	fn next(&self, _routing_info:&RoutingInfo, topology:&dyn Topology, current_router:usize, target_server:usize, num_virtual_channels:usize, _rng: &RefCell<StdRng>) -> Vec<CandidateEgress>
+	fn next(&self, _routing_info:&RoutingInfo, topology:&dyn Topology, current_router:usize, target_server:usize, num_virtual_channels:usize, _rng: &RefCell<StdRng>) -> RoutingNextCandidates
 	{
 		let (target_location,_link_class)=topology.server_neighbour(target_server);
 		let target_router=match target_location
@@ -354,7 +396,8 @@ impl Routing for Shortest
 					if server==target_server
 					{
 						//return (0..num_virtual_channels).map(|vc|(i,vc)).collect();
-						return (0..num_virtual_channels).map(|vc|CandidateEgress::new(i,vc)).collect();
+						//return (0..num_virtual_channels).map(|vc|CandidateEgress::new(i,vc)).collect();
+						return RoutingNextCandidates{candidates:(0..num_virtual_channels).map(|vc|CandidateEgress::new(i,vc)).collect(),idempotent:true};
 					}
 				}
 			}
@@ -379,7 +422,7 @@ impl Routing for Shortest
 			}
 		}
 		//println!("From router {} to router {} distance={} cand={}",current_router,target_router,distance,r.len());
-		r
+		RoutingNextCandidates{candidates:r,idempotent:true}
 	}
 	fn initialize_routing_info(&self, _routing_info:&RefCell<RoutingInfo>, _topology:&dyn Topology, _current_router:usize, _target_server:usize, _rng: &RefCell<StdRng>)
 	{
@@ -455,7 +498,7 @@ pub struct Valiant
 
 impl Routing for Valiant
 {
-	fn next(&self, routing_info:&RoutingInfo, topology:&dyn Topology, current_router:usize, target_server:usize, num_virtual_channels:usize, rng: &RefCell<StdRng>) -> Vec<CandidateEgress>
+	fn next(&self, routing_info:&RoutingInfo, topology:&dyn Topology, current_router:usize, target_server:usize, num_virtual_channels:usize, rng: &RefCell<StdRng>) -> RoutingNextCandidates
 	{
 		let (target_location,_link_class)=topology.server_neighbour(target_server);
 		let target_router=match target_location
@@ -474,7 +517,8 @@ impl Routing for Valiant
 					if server==target_server
 					{
 						//return (0..num_virtual_channels).map(|vc|(i,vc)).collect();
-						return (0..num_virtual_channels).map(|vc|CandidateEgress::new(i,vc)).collect();
+						//return (0..num_virtual_channels).map(|vc|CandidateEgress::new(i,vc)).collect();
+						return RoutingNextCandidates{candidates:(0..num_virtual_channels).map(|vc|CandidateEgress::new(i,vc)).collect(),idempotent:true}
 					}
 				}
 			}
@@ -486,7 +530,10 @@ impl Routing for Valiant
 			None =>
 			{
 				//self.second.next(&meta[1].borrow(),topology,current_router,target_server,num_virtual_channels,rng)
-				self.second.next(&meta[1].borrow(),topology,current_router,target_server,num_virtual_channels,rng).into_iter().filter(|egress|!self.first_reserved_virtual_channels.contains(&egress.virtual_channel)).collect()
+				let base=self.second.next(&meta[1].borrow(),topology,current_router,target_server,num_virtual_channels,rng);
+				let idempotent = base.idempotent;
+				let r=base.into_iter().filter(|egress|!self.first_reserved_virtual_channels.contains(&egress.virtual_channel)).collect();
+				RoutingNextCandidates{candidates:r,idempotent}
 			}
 			Some(ref s) =>
 			{
@@ -506,7 +553,9 @@ impl Routing for Valiant
 				};
 				let second_distance=topology.distance(middle,target_router);//Only exact if the base routing is shortest.
 				//self.first.next(&meta[0].borrow(),topology,current_router,middle_server,num_virtual_channels,rng).into_iter().filter(|egress|!self.second_reserved_virtual_channels.contains(&egress.virtual_channel)).collect()
-				self.first.next(&meta[0].borrow(),topology,current_router,middle_server,num_virtual_channels,rng).into_iter().filter_map(|mut egress|{
+				let base = self.first.next(&meta[0].borrow(),topology,current_router,middle_server,num_virtual_channels,rng);
+				let idempotent = base.idempotent;
+				let r=base.into_iter().filter_map(|mut egress|{
 					if self.second_reserved_virtual_channels.contains(&egress.virtual_channel) { None } else {
 						if let Some(ref mut eh)=egress.estimated_remaining_hops
 						{
@@ -514,7 +563,8 @@ impl Routing for Valiant
 						}
 						Some(egress)
 					}
-				}).collect()
+				}).collect();
+				RoutingNextCandidates{candidates:r,idempotent}
 			}
 		}
 		// let num_ports=topology.ports(current_router);
@@ -740,7 +790,7 @@ impl<R:SourceRouting + Debug> InstantiableSourceRouting for R {}
 
 impl<R:SourceRouting+Debug> Routing for R
 {
-	fn next(&self, routing_info:&RoutingInfo, topology:&dyn Topology, current_router:usize, target_server:usize, num_virtual_channels:usize, _rng: &RefCell<StdRng>) -> Vec<CandidateEgress>
+	fn next(&self, routing_info:&RoutingInfo, topology:&dyn Topology, current_router:usize, target_server:usize, num_virtual_channels:usize, _rng: &RefCell<StdRng>) -> RoutingNextCandidates
 	{
 		let (target_location,_link_class)=topology.server_neighbour(target_server);
 		let target_router=match target_location
@@ -759,7 +809,8 @@ impl<R:SourceRouting+Debug> Routing for R
 					if server==target_server
 					{
 						//return (0..num_virtual_channels).map(|vc|(i,vc)).collect();
-						return (0..num_virtual_channels).map(|vc|CandidateEgress::new(i,vc)).collect();
+						//return (0..num_virtual_channels).map(|vc|CandidateEgress::new(i,vc)).collect();
+						return RoutingNextCandidates{candidates:(0..num_virtual_channels).map(|vc|CandidateEgress::new(i,vc)).collect(),idempotent:true};
 					}
 				}
 			}
@@ -788,7 +839,7 @@ impl<R:SourceRouting+Debug> Routing for R
 			}
 		}
 		//println!("From router {} to router {} distance={} cand={}",current_router,target_router,distance,r.len());
-		r
+		RoutingNextCandidates{candidates:r,idempotent:true}
 	}
 	fn initialize_routing_info(&self, routing_info:&RefCell<RoutingInfo>, topology:&dyn Topology, current_router:usize, target_server:usize, rng: &RefCell<StdRng>)
 	{
@@ -843,6 +894,11 @@ pub enum SumRoutingPolicy
 	TryBoth,
 	Stubborn,
 	StubbornWhenSecond,
+	///Note that both routings are informed of the hops given, which could be illegal for one of them.
+	SecondWhenFirstEmpty,
+	///At every hop of the first routing give the possibility to use the second routing from the current router towards the target router.
+	///once a hop exclussive to the second routing is given continues that way.
+	EscapeToSecond,
 }
 
 pub fn new_sum_routing_policy(cv: &ConfigurationValue) -> SumRoutingPolicy
@@ -855,8 +911,8 @@ pub fn new_sum_routing_policy(cv: &ConfigurationValue) -> SumRoutingPolicy
 			"TryBoth" => SumRoutingPolicy::TryBoth,
 			"Stubborn" => SumRoutingPolicy::Stubborn,
 			"StubbornWhenSecond" => SumRoutingPolicy::StubbornWhenSecond,
-			//"Shortest" => SumRoutingPolicy::Shortest,
-			//"Hops" => SumRoutingPolicy::Hops,
+			"SecondWhenFirstEmpty" => SumRoutingPolicy::SecondWhenFirstEmpty,
+			"EscapeToSecond" => SumRoutingPolicy::EscapeToSecond,
 			_ => panic!("Unknown sum routing policy {}",cv_name),
 		}
 	}
@@ -888,7 +944,7 @@ pub struct SumRouting
 //* [a,b,c] if a request by routing c has been made, but the two routing are still available.
 impl Routing for SumRouting
 {
-	fn next(&self, routing_info:&RoutingInfo, topology:&dyn Topology, current_router:usize, target_server:usize, num_virtual_channels:usize, rng: &RefCell<StdRng>) -> Vec<CandidateEgress>
+	fn next(&self, routing_info:&RoutingInfo, topology:&dyn Topology, current_router:usize, target_server:usize, num_virtual_channels:usize, rng: &RefCell<StdRng>) -> RoutingNextCandidates
 	{
 		let (target_location,_link_class)=topology.server_neighbour(target_server);
 		let target_router=match target_location
@@ -907,14 +963,15 @@ impl Routing for SumRouting
 					if server==target_server
 					{
 						//return (0..num_virtual_channels).map(|vc|(i,vc)).collect();
-						return (0..num_virtual_channels).map(|vc|CandidateEgress::new(i,vc)).collect();
+						//return (0..num_virtual_channels).map(|vc|CandidateEgress::new(i,vc)).collect();
+						return RoutingNextCandidates{candidates:(0..num_virtual_channels).map(|vc|CandidateEgress::new(i,vc)).collect(),idempotent:true};
 					}
 				}
 			}
 			unreachable!();
 		}
 		let meta=routing_info.meta.as_ref().unwrap();
-		match routing_info.selections
+		let r = match routing_info.selections
 		{
 			None =>
 			{
@@ -938,7 +995,15 @@ impl Routing for SumRouting
 					let el1=self.extra_label[1];
 					//let r1=self.second_routing.next(&meta[1].borrow(),topology,current_router,target_server,avc1.len(),rng).into_iter().map( |candidate| CandidateEgress{virtual_channel:avc1[candidate.virtual_channel],label:candidate.label+el1,annotation:Some(RoutingAnnotation{values:vec![1],meta:vec![candidate.annotation]}),..candidate} );
 					let r1=self.routing[1].next(&meta[1].borrow(),topology,current_router,target_server,avc1.len(),rng).into_iter().map( |candidate| CandidateEgress{virtual_channel:avc1[candidate.virtual_channel],label:candidate.label+el1,annotation:Some(RoutingAnnotation{values:vec![1],meta:vec![candidate.annotation]}),..candidate} );
-					r0.chain(r1).collect()
+					match self.policy
+					{
+						SumRoutingPolicy::SecondWhenFirstEmpty =>
+						{
+							let r : Vec<_> =r0.collect();
+							if r.is_empty() { r1.collect() } else { r }
+						}
+						_ => r0.chain(r1).collect()
+					}
 				}
 				else
 				{
@@ -956,14 +1021,17 @@ impl Routing for SumRouting
 					.map( |candidate| CandidateEgress{virtual_channel:allowed_virtual_channels[candidate.virtual_channel],label:candidate.label+extra_label,..candidate} ).collect()
 				}
 			}
-		}
+		};
+		//FIXME: we can recover idempotence in some cases.
+		RoutingNextCandidates{candidates:r,idempotent:false}
 	}
 	fn initialize_routing_info(&self, routing_info:&RefCell<RoutingInfo>, topology:&dyn Topology, current_router:usize, target_server:usize, rng: &RefCell<StdRng>)
 	{
 		let all:Vec<i32> = match self.policy
 		{
 			SumRoutingPolicy::Random => vec![rng.borrow_mut().gen_range(0,2)],
-			SumRoutingPolicy::TryBoth | SumRoutingPolicy::Stubborn | SumRoutingPolicy::StubbornWhenSecond => vec![0,1],
+			SumRoutingPolicy::TryBoth | SumRoutingPolicy::Stubborn | SumRoutingPolicy::StubbornWhenSecond
+			| SumRoutingPolicy::SecondWhenFirstEmpty | SumRoutingPolicy::EscapeToSecond => vec![0,1],
 		};
 		let mut bri=routing_info.borrow_mut();
 		//bri.meta=Some(vec![RefCell::new(RoutingInfo::new()),RefCell::new(RoutingInfo::new())]);
@@ -979,22 +1047,48 @@ impl Routing for SumRouting
 	fn update_routing_info(&self, routing_info:&RefCell<RoutingInfo>, topology:&dyn Topology, current_router:usize, current_port:usize, target_server:usize, rng: &RefCell<StdRng>)
 	{
 		let mut bri=routing_info.borrow_mut();
-		let s=match bri.selections
+		let mut cs = match bri.selections
 		{
 			None => unreachable!(),
-			Some(ref t) => if t.len()==1 {
-				t[0] as usize
-			} else {
-				let s=t[2];
-				bri.selections=Some(vec![s]);
-				s as usize
+			Some(ref t) =>
+			{
+				if t.len()==3 {
+					match self.policy
+					{
+						SumRoutingPolicy::SecondWhenFirstEmpty => t.clone(),
+						_ => vec![t[2]],
+						//let s=t[2];
+						//bri.selections=Some(vec![s]);
+						//s as usize
+					}
+				} else { t.clone() }
+
 			},
 		};
-		//let routing=if s==0 { &self.first_routing } else { &self.second_routing };
-		let routing = &self.routing[s];
-		let meta=bri.meta.as_mut().unwrap();
-		meta[s].borrow_mut().hops+=1;
-		routing.update_routing_info(&meta[s],topology,current_router,current_port,target_server,rng);
+		for &is in cs.iter()
+		{
+			let s = is as usize;
+			let routing = &self.routing[s];
+			let meta=bri.meta.as_mut().unwrap();
+			meta[s].borrow_mut().hops+=1;
+			routing.update_routing_info(&meta[s],topology,current_router,current_port,target_server,rng);
+		}
+		if let SumRoutingPolicy::EscapeToSecond = self.policy
+		{
+			if cs[0]==0
+			{
+				//Readd the escape option
+				cs = vec![0,1];
+				let second_meta = RefCell::new(RoutingInfo::new());
+				self.routing[1].initialize_routing_info(&second_meta,topology,current_router,target_server,rng);
+				match bri.meta
+				{
+					Some(ref mut a) => a[1] = second_meta,
+					_ => panic!("No meta data for EscapeToSecond"),
+				};
+			}
+		}
+		bri.selections=Some(cs);
 	}
 	fn initialize(&mut self, topology:&Box<dyn Topology>, rng: &RefCell<StdRng>)
 	{
@@ -1129,7 +1223,7 @@ pub struct Mindless
 
 impl Routing for Mindless
 {
-	fn next(&self, _routing_info:&RoutingInfo, topology:&dyn Topology, current_router:usize, target_server:usize, num_virtual_channels:usize, _rng: &RefCell<StdRng>) -> Vec<CandidateEgress>
+	fn next(&self, _routing_info:&RoutingInfo, topology:&dyn Topology, current_router:usize, target_server:usize, num_virtual_channels:usize, _rng: &RefCell<StdRng>) -> RoutingNextCandidates
 	{
 		let (target_location,_link_class)=topology.server_neighbour(target_server);
 		let target_router=match target_location
@@ -1147,7 +1241,8 @@ impl Routing for Mindless
 					if server==target_server
 					{
 						//return (0..num_virtual_channels).map(|vc|(i,vc)).collect();
-						return (0..num_virtual_channels).map(|vc|CandidateEgress::new(i,vc)).collect();
+						//return (0..num_virtual_channels).map(|vc|CandidateEgress::new(i,vc)).collect();
+						return RoutingNextCandidates{candidates:(0..num_virtual_channels).map(|vc|CandidateEgress::new(i,vc)).collect(),idempotent:true}
 					}
 				}
 			}
@@ -1164,7 +1259,7 @@ impl Routing for Mindless
 				r.extend((0..num_virtual_channels).map(|vc|CandidateEgress::new(i,vc)));
 			}
 		}
-		r
+		RoutingNextCandidates{candidates:r,idempotent:true}
 	}
 	fn initialize_routing_info(&self, _routing_info:&RefCell<RoutingInfo>, _topology:&dyn Topology, _current_router:usize, _target_server:usize, _rng: &RefCell<StdRng>)
 	{
@@ -1230,7 +1325,7 @@ pub struct WeighedShortest
 
 impl Routing for WeighedShortest
 {
-	fn next(&self, _routing_info:&RoutingInfo, topology:&dyn Topology, current_router:usize, target_server:usize, num_virtual_channels:usize, _rng: &RefCell<StdRng>) -> Vec<CandidateEgress>
+	fn next(&self, _routing_info:&RoutingInfo, topology:&dyn Topology, current_router:usize, target_server:usize, num_virtual_channels:usize, _rng: &RefCell<StdRng>) -> RoutingNextCandidates
 	{
 		let (target_location,_link_class)=topology.server_neighbour(target_server);
 		let target_router=match target_location
@@ -1252,7 +1347,8 @@ impl Routing for WeighedShortest
 					if server==target_server
 					{
 						//return (0..num_virtual_channels).map(|vc|(i,vc)).collect();
-						return (0..num_virtual_channels).map(|vc|CandidateEgress::new(i,vc)).collect();
+						//return (0..num_virtual_channels).map(|vc|CandidateEgress::new(i,vc)).collect();
+						return RoutingNextCandidates{candidates:(0..num_virtual_channels).map(|vc|CandidateEgress::new(i,vc)).collect(),idempotent:true};
 					}
 				}
 			}
@@ -1280,7 +1376,7 @@ impl Routing for WeighedShortest
 			}
 		}
 		//println!("From router {} to router {} distance={} cand={}",current_router,target_router,distance,r.len());
-		r
+		RoutingNextCandidates{candidates:r,idempotent:true}
 	}
 	//fn initialize_routing_info(&self, routing_info:&mut RoutingInfo, toology:&dyn Topology, current_router:usize, target_server:usize)
 	fn initialize_routing_info(&self, _routing_info:&RefCell<RoutingInfo>, _topology:&dyn Topology, _current_router:usize, _target_server:usize, _rng: &RefCell<StdRng>)
@@ -1353,6 +1449,7 @@ impl WeighedShortest
 ///Wraps a routing so that only one request is made in every router.
 ///The first time the router make a port request, that request is stored and repeated in further calls to `next` until reaching a new router.
 ///Stores port, virtual_channel, label into routing_info.selections.
+///Note that has `idempotent=false` since the value may change if the request has not actually been made.
 #[derive(Debug)]
 pub struct Stubborn
 {
@@ -1361,7 +1458,7 @@ pub struct Stubborn
 
 impl Routing for Stubborn
 {
-	fn next(&self, routing_info:&RoutingInfo, topology:&dyn Topology, current_router:usize, target_server:usize, num_virtual_channels:usize, rng: &RefCell<StdRng>) -> Vec<CandidateEgress>
+	fn next(&self, routing_info:&RoutingInfo, topology:&dyn Topology, current_router:usize, target_server:usize, num_virtual_channels:usize, rng: &RefCell<StdRng>) -> RoutingNextCandidates
 	{
 		let (target_location,_link_class)=topology.server_neighbour(target_server);
 		let target_router=match target_location
@@ -1379,7 +1476,8 @@ impl Routing for Stubborn
 					if server==target_server
 					{
 						//return (0..num_virtual_channels).map(|vc|(i,vc)).collect();
-						return (0..num_virtual_channels).map(|vc|CandidateEgress::new(i,vc)).collect();
+						//return (0..num_virtual_channels).map(|vc|CandidateEgress::new(i,vc)).collect();
+						return RoutingNextCandidates{candidates:(0..num_virtual_channels).map(|vc|CandidateEgress::new(i,vc)).collect(),idempotent:true};
 					}
 				}
 			}
@@ -1387,10 +1485,12 @@ impl Routing for Stubborn
 		}
 		if let Some(ref sel)=routing_info.selections
 		{
-			return vec![CandidateEgress{port:sel[0] as usize,virtual_channel:sel[1] as usize,label:sel[2],..Default::default()}]
+			//return vec![CandidateEgress{port:sel[0] as usize,virtual_channel:sel[1] as usize,label:sel[2],..Default::default()}]
+			return RoutingNextCandidates{candidates:vec![CandidateEgress{port:sel[0] as usize,virtual_channel:sel[1] as usize,label:sel[2],..Default::default()}],idempotent:false};
 		}
 		//return self.routing.next(&routing_info.meta.as_ref().unwrap()[0].borrow(),topology,current_router,target_server,num_virtual_channels,rng)
-		return self.routing.next(&routing_info.meta.as_ref().unwrap()[0].borrow(),topology,current_router,target_server,num_virtual_channels,rng).into_iter().map(|candidate|CandidateEgress{annotation:Some(RoutingAnnotation{values:vec![candidate.label],meta:vec![candidate.annotation]}),..candidate}).collect()
+		//return self.routing.next(&routing_info.meta.as_ref().unwrap()[0].borrow(),topology,current_router,target_server,num_virtual_channels,rng).into_iter().map(|candidate|CandidateEgress{annotation:Some(RoutingAnnotation{values:vec![candidate.label],meta:vec![candidate.annotation]}),..candidate}).collect()
+		return RoutingNextCandidates{candidates:self.routing.next(&routing_info.meta.as_ref().unwrap()[0].borrow(),topology,current_router,target_server,num_virtual_channels,rng).into_iter().map(|candidate|CandidateEgress{annotation:Some(RoutingAnnotation{values:vec![candidate.label],meta:vec![candidate.annotation]}),..candidate}).collect(),idempotent:false}
 	}
 	fn initialize_routing_info(&self, routing_info:&RefCell<RoutingInfo>, topology:&dyn Topology, current_router:usize, target_server:usize, rng: &RefCell<StdRng>)
 	{
@@ -1479,7 +1579,7 @@ pub struct UpDown
 
 impl Routing for UpDown
 {
-	fn next(&self, _routing_info:&RoutingInfo, topology:&dyn Topology, current_router:usize, target_server:usize, num_virtual_channels:usize, _rng: &RefCell<StdRng>) -> Vec<CandidateEgress>
+	fn next(&self, _routing_info:&RoutingInfo, topology:&dyn Topology, current_router:usize, target_server:usize, num_virtual_channels:usize, _rng: &RefCell<StdRng>) -> RoutingNextCandidates
 	{
 		let (target_location,_link_class)=topology.server_neighbour(target_server);
 		let target_router=match target_location
@@ -1498,7 +1598,8 @@ impl Routing for UpDown
 					if server==target_server
 					{
 						//return (0..num_virtual_channels).map(|vc|(i,vc)).collect();
-						return (0..num_virtual_channels).map(|vc|CandidateEgress::new(i,vc)).collect();
+						//return (0..num_virtual_channels).map(|vc|CandidateEgress::new(i,vc)).collect();
+						return RoutingNextCandidates{candidates:(0..num_virtual_channels).map(|vc|CandidateEgress::new(i,vc)).collect(),idempotent:true};
 					}
 				}
 			}
@@ -1521,7 +1622,7 @@ impl Routing for UpDown
 			}
 		}
 		//println!("From router {} to router {} distance={} cand={}",current_router,target_router,distance,r.len());
-		r
+		RoutingNextCandidates{candidates:r,idempotent:true}
 	}
 	fn initialize_routing_info(&self, _routing_info:&RefCell<RoutingInfo>, _topology:&dyn Topology, _current_router:usize, _target_server:usize, _rng: &RefCell<StdRng>)
 	{
@@ -1598,7 +1699,7 @@ pub struct ExplicitUpDown
 
 impl Routing for ExplicitUpDown
 {
-	fn next(&self, _routing_info:&RoutingInfo, topology:&dyn Topology, current_router:usize, target_server:usize, num_virtual_channels:usize, _rng: &RefCell<StdRng>) -> Vec<CandidateEgress>
+	fn next(&self, _routing_info:&RoutingInfo, topology:&dyn Topology, current_router:usize, target_server:usize, num_virtual_channels:usize, _rng: &RefCell<StdRng>) -> RoutingNextCandidates
 	{
 		let (target_location,_link_class)=topology.server_neighbour(target_server);
 		let target_router=match target_location
@@ -1617,7 +1718,8 @@ impl Routing for ExplicitUpDown
 					if server==target_server
 					{
 						//return (0..num_virtual_channels).map(|vc|(i,vc)).collect();
-						return (0..num_virtual_channels).map(|vc|CandidateEgress::new(i,vc)).collect();
+						//return (0..num_virtual_channels).map(|vc|CandidateEgress::new(i,vc)).collect();
+						return RoutingNextCandidates{candidates:(0..num_virtual_channels).map(|vc|CandidateEgress::new(i,vc)).collect(),idempotent:true};
 					}
 				}
 			}
@@ -1640,7 +1742,7 @@ impl Routing for ExplicitUpDown
 			}
 		}
 		//println!("From router {} to router {} distance={} cand={}",current_router,target_router,distance,r.len());
-		r
+		RoutingNextCandidates{candidates:r,idempotent:true}
 	}
 	fn initialize_routing_info(&self, _routing_info:&RefCell<RoutingInfo>, _topology:&dyn Topology, _current_router:usize, _target_server:usize, _rng: &RefCell<StdRng>)
 	{
@@ -1786,19 +1888,21 @@ pub struct ChannelsPerHop
 {
 	///The base routing to use.
 	routing: Box<dyn Routing>,
-	///channels[k] is the list of available VCs to use in the k-th hop.
+	///`channels[k]` is the list of available VCs to use in the `k`-th hop.
 	///This includes the last hop towards the server.
 	channels: Vec<Vec<usize>>,
 }
 
 impl Routing for ChannelsPerHop
 {
-	fn next(&self, routing_info:&RoutingInfo, topology:&dyn Topology, current_router:usize, target_server:usize, num_virtual_channels:usize, rng: &RefCell<StdRng>) -> Vec<CandidateEgress>
+	fn next(&self, routing_info:&RoutingInfo, topology:&dyn Topology, current_router:usize, target_server:usize, num_virtual_channels:usize, rng: &RefCell<StdRng>) -> RoutingNextCandidates
 	{
 		//println!("{}",topology.diameter());
 		let vcs = &self.channels[routing_info.hops];
 		let candidates = self.routing.next(routing_info,topology,current_router,target_server,num_virtual_channels,rng);
-		candidates.into_iter().filter(|c|vcs.contains(&c.virtual_channel)).collect()
+		let idempotent = candidates.idempotent;
+		let r = candidates.into_iter().filter(|c|vcs.contains(&c.virtual_channel)).collect();
+		RoutingNextCandidates{candidates:r,idempotent}
 	}
 	fn initialize_routing_info(&self, routing_info:&RefCell<RoutingInfo>, topology:&dyn Topology, current_router:usize, target_server:usize, rng: &RefCell<StdRng>)
 	{
@@ -1880,18 +1984,19 @@ pub struct ChannelsPerHopPerLinkClass
 {
 	///The base routing to use.
 	routing: Box<dyn Routing>,
-	///channels[class][k] is the list of available VCs to use in the k-th hop given in links of the given `class`.
+	///`channels[class][k]` is the list of available VCs to use in the k-th hop given in links of the given `class`.
 	channels: Vec<Vec<Vec<usize>>>,
 }
 
 impl Routing for ChannelsPerHopPerLinkClass
 {
-	fn next(&self, routing_info:&RoutingInfo, topology:&dyn Topology, current_router:usize, target_server:usize, num_virtual_channels:usize, rng: &RefCell<StdRng>) -> Vec<CandidateEgress>
+	fn next(&self, routing_info:&RoutingInfo, topology:&dyn Topology, current_router:usize, target_server:usize, num_virtual_channels:usize, rng: &RefCell<StdRng>) -> RoutingNextCandidates
 	{
 		//println!("{}",topology.diameter());
 		let candidates = self.routing.next(&routing_info.meta.as_ref().unwrap()[0].borrow(),topology,current_router,target_server,num_virtual_channels,rng);
+		let idempotent = candidates.idempotent;
 		let hops = &routing_info.selections.as_ref().unwrap();
-		candidates.into_iter().filter(|c|{
+		let r = candidates.into_iter().filter(|c|{
 			let (_next_location,link_class)=topology.neighbour(current_router,c.port);
 			let h = hops[link_class] as usize;
 			//println!("h={} link_class={} channels={:?}",h,link_class,self.channels[link_class]);
@@ -1901,7 +2006,8 @@ impl Routing for ChannelsPerHopPerLinkClass
 			}
 			//self.channels[link_class].len()>h && self.channels[link_class][h].contains(&c.virtual_channel)
 			self.channels[link_class][h].contains(&c.virtual_channel)
-		}).collect()
+		}).collect();
+		RoutingNextCandidates{candidates:r,idempotent}
 	}
 	fn initialize_routing_info(&self, routing_info:&RefCell<RoutingInfo>, topology:&dyn Topology, current_router:usize, target_server:usize, rng: &RefCell<StdRng>)
 	{
@@ -2005,19 +2111,21 @@ pub struct AscendantChannelsWithLinkClass
 
 impl Routing for AscendantChannelsWithLinkClass
 {
-	fn next(&self, routing_info:&RoutingInfo, topology:&dyn Topology, current_router:usize, target_server:usize, num_virtual_channels:usize, rng: &RefCell<StdRng>) -> Vec<CandidateEgress>
+	fn next(&self, routing_info:&RoutingInfo, topology:&dyn Topology, current_router:usize, target_server:usize, num_virtual_channels:usize, rng: &RefCell<StdRng>) -> RoutingNextCandidates
 	{
 		//println!("{}",topology.diameter());
 		let candidates = self.routing.next(&routing_info.meta.as_ref().unwrap()[0].borrow(),topology,current_router,target_server,num_virtual_channels,rng);
+		let idempotent = candidates.idempotent;
 		let hops_since = &routing_info.selections.as_ref().unwrap();
-		candidates.into_iter().filter(|c|{
+		let r = candidates.into_iter().filter(|c|{
 			let (_next_location,link_class)=topology.neighbour(current_router,c.port);
 			if link_class>= self.bases.len() { return true; }
 			//let h = hops_since[link_class] as usize;
 			let vc = (link_class..self.bases.len()).rev().fold(0, |x,class| x*self.bases[class]+(hops_since[class] as usize) );
 			//if link_class==0 && vc!=hops_since[1] as usize{ println!("hops_since={:?} link_class={} vc={}",hops_since,link_class,vc); }
 			c.virtual_channel == vc
-		}).collect()
+		}).collect();
+		RoutingNextCandidates{candidates:r,idempotent}
 	}
 	fn initialize_routing_info(&self, routing_info:&RefCell<RoutingInfo>, topology:&dyn Topology, current_router:usize, target_server:usize, rng: &RefCell<StdRng>)
 	{
@@ -2120,11 +2228,12 @@ pub struct ChannelMap
 
 impl Routing for ChannelMap
 {
-	fn next(&self, routing_info:&RoutingInfo, topology:&dyn Topology, current_router:usize, target_server:usize, _num_virtual_channels:usize, rng: &RefCell<StdRng>) -> Vec<CandidateEgress>
+	fn next(&self, routing_info:&RoutingInfo, topology:&dyn Topology, current_router:usize, target_server:usize, _num_virtual_channels:usize, rng: &RefCell<StdRng>) -> RoutingNextCandidates
 	{
 		//println!("{}",topology.diameter());
 		//let vcs = &self.channels[routing_info.hops];
 		let candidates = self.routing.next(routing_info,topology,current_router,target_server,self.map.len(),rng);
+		let idempotent = candidates.idempotent;
 		//candidates.into_iter().filter(|c|vcs.contains(&c.virtual_channel)).collect()
 		let mut r=Vec::with_capacity(candidates.len());
 		for can in candidates.into_iter()
@@ -2136,7 +2245,7 @@ impl Routing for ChannelMap
 				r.push(new);
 			}
 		}
-		r
+		RoutingNextCandidates{candidates:r,idempotent}
 	}
 	fn initialize_routing_info(&self, routing_info:&RefCell<RoutingInfo>, topology:&dyn Topology, current_router:usize, target_server:usize, rng: &RefCell<StdRng>)
 	{
@@ -2224,7 +2333,7 @@ pub struct SourceAdaptiveRouting
 
 impl Routing for SourceAdaptiveRouting
 {
-	fn next(&self, routing_info:&RoutingInfo, topology:&dyn Topology, current_router:usize, target_server:usize, num_virtual_channels:usize, _rng: &RefCell<StdRng>) -> Vec<CandidateEgress>
+	fn next(&self, routing_info:&RoutingInfo, topology:&dyn Topology, current_router:usize, target_server:usize, num_virtual_channels:usize, _rng: &RefCell<StdRng>) -> RoutingNextCandidates
 	{
 		let (target_location,_link_class)=topology.server_neighbour(target_server);
 		let target_router=match target_location
@@ -2243,7 +2352,11 @@ impl Routing for SourceAdaptiveRouting
 					if server==target_server
 					{
 						//return (0..num_virtual_channels).map(|vc|(i,vc)).collect();
-						return (0..num_virtual_channels).map(|vc|CandidateEgress::new(i,vc)).collect();
+						//return (0..num_virtual_channels).map(|vc|CandidateEgress::new(i,vc)).collect();
+						return RoutingNextCandidates{
+							candidates:(0..num_virtual_channels).map(|vc|CandidateEgress::new(i,vc)).collect(),
+							idempotent:true
+						};
 					}
 				}
 			}
@@ -2278,7 +2391,7 @@ impl Routing for SourceAdaptiveRouting
 			}
 		}
 		//println!("From router {} to router {} distance={} cand={}",current_router,target_router,distance,r.len());
-		r
+		RoutingNextCandidates{candidates:r,idempotent:true}
 	}
 	fn initialize_routing_info(&self, routing_info:&RefCell<RoutingInfo>, topology:&dyn Topology, current_router:usize, target_server:usize, rng: &RefCell<StdRng>)
 	{
@@ -2368,7 +2481,7 @@ pub struct EachLengthSourceAdaptiveRouting
 
 impl Routing for EachLengthSourceAdaptiveRouting
 {
-	fn next(&self, routing_info:&RoutingInfo, topology:&dyn Topology, current_router:usize, target_server:usize, num_virtual_channels:usize, _rng: &RefCell<StdRng>) -> Vec<CandidateEgress>
+	fn next(&self, routing_info:&RoutingInfo, topology:&dyn Topology, current_router:usize, target_server:usize, num_virtual_channels:usize, _rng: &RefCell<StdRng>) -> RoutingNextCandidates
 	{
 		let (target_location,_link_class)=topology.server_neighbour(target_server);
 		let target_router=match target_location
@@ -2387,7 +2500,11 @@ impl Routing for EachLengthSourceAdaptiveRouting
 					if server==target_server
 					{
 						//return (0..num_virtual_channels).map(|vc|(i,vc)).collect();
-						return (0..num_virtual_channels).map(|vc|CandidateEgress::new(i,vc)).collect();
+						//return (0..num_virtual_channels).map(|vc|CandidateEgress::new(i,vc)).collect();
+						return RoutingNextCandidates{
+							candidates:(0..num_virtual_channels).map(|vc|CandidateEgress::new(i,vc)).collect(),
+							idempotent:true
+						};
 					}
 				}
 			}
@@ -2423,7 +2540,7 @@ impl Routing for EachLengthSourceAdaptiveRouting
 			}
 		}
 		//println!("From router {} to router {} distance={} cand={}",current_router,target_router,distance,r.len());
-		r
+		RoutingNextCandidates{candidates:r,idempotent:true}
 	}
 	fn initialize_routing_info(&self, routing_info:&RefCell<RoutingInfo>, topology:&dyn Topology, current_router:usize, target_server:usize, rng: &RefCell<StdRng>)
 	{
@@ -2501,3 +2618,4 @@ impl Routing for EachLengthSourceAdaptiveRouting
 	{
 	}
 }
+
