@@ -408,7 +408,7 @@ pub trait Topology : Quantifiable + std::fmt::Debug
 	}
 
 	/// Find the coponents of the subtopology induced via the allowed links.
-	/// Returns vector ret with ret[k] containing the vertices in the k-th component.
+	/// Returns vector `ret` with `ret[k]` containing the vertices in the `k`-th component.
 	fn components(&self,allowed_classes:&[bool]) -> Vec<Vec<usize>>
 	{
 		let mut r=vec![];
@@ -437,10 +437,10 @@ pub trait Topology : Quantifiable + std::fmt::Debug
 		return r;
 	}
 	
-	/// returns a cople matrices (N,F) with
-	///	N[u,v] = number of neighbours w of v with D(u,v)>D(u,w).
-	///	F[u,v] = number of neighbours w of v with D(u,v)<D(u,w).
-	/// A router v with F[u,v]=0 is called a boundary vertex of u.
+	/// returns a couple matrices `(N,F)` with
+	///	`N[u,v]` = number of neighbours w of v with `D(u,v)>D(u,w)`.
+	///	`F[u,v]` = number of neighbours w of v with `D(u,v)<D(u,w)`.
+	/// A router `v` with `F[u,v]=0` is called a boundary vertex of u.
 	fn compute_near_far_matrices(&self) -> (Matrix<usize>,Matrix<usize>)
 	{
 		let n=self.num_routers();
@@ -515,6 +515,8 @@ pub trait Topology : Quantifiable + std::fmt::Debug
 	{
 		let n=self.num_routers();
 		let mut max_link_class=0;
+		let min_deg= self.minimum_degree();
+		let max_deg= self.maximum_degree();
 		for router_index in 0..n
 		{
 			let deg = self.degree(router_index);
@@ -562,9 +564,9 @@ pub trait Topology : Quantifiable + std::fmt::Debug
 						{
 							panic!("port {} at router {} has non-matching link class {} vs {}",port_index,router_index,link_class,rev_link_class);
 						}
-						if port_index>=deg
+						if port_index>=max_deg
 						{
-							panic!("port {} at router {} connects to another router and it is >=degree={}",port_index,router_index,deg);
+							panic!("port {} at router {} connects to another router and it is >=maximum_degree={}>=degree={}",port_index,router_index,max_deg,deg);
 						}
 					},
 					Location::ServerPort(server_index) =>
@@ -588,9 +590,9 @@ pub trait Topology : Quantifiable + std::fmt::Debug
 						{
 							panic!("port {} at router {} has non-matching link class {} vs {}",port_index,router_index,link_class,rev_link_class);
 						}
-						if port_index<deg
+						if port_index<min_deg
 						{
-							panic!("port {} at router {} connects to a server and it is <degree={}",port_index,router_index,deg);
+							panic!("port {} at router {} connects to a server and it is <minimum_degree={}<=degree={}",port_index,router_index,min_deg,deg);
 						}
 					},
 					Location::None => println!("WARNING: disconnected port {} at router {}",port_index,router_index),
@@ -641,7 +643,7 @@ Build a topology. All topologies should admit an optional `legend_name` to be us
 ## Cartesian topologies
 
 ### Mesh example: a bidimensional mesh of side 16. Routers in the periphery has less degree, defined as un-connected ports.
-```
+```ignore
 Mesh{
 	sides: [16,16],
 	servers_per_router:1,
